"""Network client for supervisor."""

from .client import _SupervisorComponentClient
from .models.network import (
    AccessPoint,
    AccessPointList,
    NetworkInfo,
    NetworkInterface,
    NetworkInterfaceConfig,
    VlanConfig,
)


class NetworkClient(_SupervisorComponentClient):
    """Handles network access in supervisor."""

    async def info(self) -> NetworkInfo:
        """Get network info."""
        result = await self._client.get("network/info")
        return NetworkInfo.from_dict(result.data)

    async def reload(self) -> None:
        """Reload network info caches."""
        await self._client.post("network/reload")

    async def interface_info(self, interface: str) -> NetworkInterface:
        """Get network interface info."""
        result = await self._client.get(f"network/interface/{interface}/info")
        return NetworkInterface.from_dict(result.data)

    async def update_interface(
        self, interface: str, config: NetworkInterfaceConfig
    ) -> None:
        """Update a network interface."""
        await self._client.post(
            f"network/interface/{interface}/update", json=config.to_dict()
        )

    async def access_points(self, interface: str) -> list[AccessPoint]:
        """Get access points visible to a wireless interface."""
        result = await self._client.get(f"network/interface/{interface}/accesspoints")
        return AccessPointList.from_dict(result.data).accesspoints

    async def save_vlan(
        self, interface: str, vlan: int, config: VlanConfig | None = None
    ) -> None:
        """Create or update a vlan for an ethernet interface."""
        await self._client.post(
            f"network/interface/{interface}/vlan/{vlan}",
            json=config.to_dict() if config else None,
        )



"""Constants for aiohasupervisor."""

from enum import StrEnum

from aiohttp import ClientTimeout

DEFAULT_TIMEOUT = ClientTimeout(total=10)
TIMEOUT_60_SECONDS = ClientTimeout(total=60)


class ResponseType(StrEnum):
    """Expected response type."""

    NONE = "none"
    JSON = "json"
    STREAM = "stream"
    TEXT = "text"



"""Supervisor client for supervisor."""

from aiohttp import ClientTimeout

from .client import _SupervisorComponentClient
from .const import ResponseType
from .models.supervisor import (
    SupervisorInfo,
    SupervisorOptions,
    SupervisorStats,
    SupervisorUpdateOptions,
)


class SupervisorManagementClient(_SupervisorComponentClient):
    """Handles supervisor access in supervisor."""

    async def ping(self) -> None:
        """Check connection to supervisor."""
        await self._client.get(
            "supervisor/ping",
            response_type=ResponseType.NONE,
            timeout=ClientTimeout(total=15),
        )

    async def info(self) -> SupervisorInfo:
        """Get supervisor info."""
        result = await self._client.get("supervisor/info")
        return SupervisorInfo.from_dict(result.data)

    async def stats(self) -> SupervisorStats:
        """Get supervisor stats."""
        result = await self._client.get("supervisor/stats")
        return SupervisorStats.from_dict(result.data)

    async def update(self, options: SupervisorUpdateOptions | None = None) -> None:
        """Update supervisor.

        Providing a target version in options only works on development systems.
        On non-development systems this API will always update supervisor to the
        latest version and ignore that field.
        """
        await self._client.post(
            "supervisor/update",
            json=options.to_dict() if options else None,
            timeout=None,
        )

    async def reload(self) -> None:
        """Reload supervisor (add-ons, configuration, etc)."""
        await self._client.post("supervisor/reload")

    async def restart(self) -> None:
        """Restart supervisor."""
        await self._client.post("supervisor/restart")

    async def set_options(self, options: SupervisorOptions) -> None:
        """Set supervisor options."""
        await self._client.post("supervisor/options", json=options.to_dict())

    async def repair(self) -> None:
        """Repair local supervisor and docker setup."""
        await self._client.post("supervisor/repair")



"""Test backups supervisor client."""

import asyncio
from collections.abc import AsyncIterator
from datetime import UTC, datetime
from pathlib import PurePath
from typing import Any

from aioresponses import CallbackResult, aioresponses
import pytest
from yarl import URL

from aiohasupervisor import SupervisorClient
from aiohasupervisor.models import (
    AddonSet,
    BackupLocationAttributes,
    BackupsOptions,
    DownloadBackupOptions,
    Folder,
    FreezeOptions,
    FullBackupOptions,
    FullRestoreOptions,
    PartialBackupOptions,
    PartialRestoreOptions,
    RemoveBackupOptions,
    UploadBackupOptions,
)

from . import load_fixture
from .const import SUPERVISOR_URL


async def test_backups_list(
    responses: aioresponses, supervisor_client: SupervisorClient
) -> None:
    """Test backups list API."""
    responses.get(
        f"{SUPERVISOR_URL}/backups", status=200, body=load_fixture("backups_list.json")
    )
    backups = await supervisor_client.backups.list()
    assert backups[0].slug == "58bc7491"
    assert backups[0].type == "full"
    assert backups[0].date == datetime(2024, 4, 6, 7, 5, 40, 0, UTC)
    assert backups[0].compressed is True
    assert backups[0].content.homeassistant is True
    assert backups[0].content.folders == ["share", "addons/local", "ssl", "media"]
    assert backups[1].slug == "69558789"
    assert backups[1].type == "partial"


async def test_backups_info(
    responses: aioresponses, supervisor_client: SupervisorClient
) -> None:
    """Test backups info API."""
    responses.get(
        f"{SUPERVISOR_URL}/backups/info",
        status=200,
        body=load_fixture("backups_info.json"),
    )
    info = await supervisor_client.backups.info()
    assert info.backups[0].slug == "58bc7491"
    assert info.backups[1].slug == "69558789"
    assert info.days_until_stale == 30


async def test_backups_options(
    responses: aioresponses, supervisor_client: SupervisorClient
) -> None:
    """Test backups options API."""
    responses.post(f"{SUPERVISOR_URL}/backups/options", status=200)
    assert (
        await supervisor_client.backups.set_options(BackupsOptions(days_until_stale=10))
        is None
    )
    assert responses.requests.keys() == {
        ("POST", URL(f"{SUPERVISOR_URL}/backups/options"))
    }


async def test_backups_reload(
    responses: aioresponses, supervisor_client: SupervisorClient
) -> None:
    """Test backups reload API."""
    responses.post(f"{SUPERVISOR_URL}/backups/reload", status=200)
    assert await supervisor_client.backups.reload() is None
    assert responses.requests.keys() == {
        ("POST", URL(f"{SUPERVISOR_URL}/backups/reload"))
    }


@pytest.mark.parametrize("options", [None, FreezeOptions(timeout=1000)])
async def test_backups_freeze(
    responses: aioresponses,
    supervisor_client: SupervisorClient,
    options: FreezeOptions | None,
) -> None:
    """Test backups freeze API."""
    responses.post(f"{SUPERVISOR_URL}/backups/freeze", status=200)
    assert await supervisor_client.backups.freeze(options) is None
    assert responses.requests.keys() == {
        ("POST", URL(f"{SUPERVISOR_URL}/backups/freeze"))
    }


async def test_backups_thaw(
    responses: aioresponses, supervisor_client: SupervisorClient
) -> None:
    """Test backups thaw API."""
    responses.post(f"{SUPERVISOR_URL}/backups/thaw", status=200)
    assert await supervisor_client.backups.thaw() is None
    assert responses.requests.keys() == {
        ("POST", URL(f"{SUPERVISOR_URL}/backups/thaw"))
    }


async def test_partial_backup_options() -> None:
    """Test partial backup options."""
    assert PartialBackupOptions(name="good", addons={"a"})
    assert PartialBackupOptions(name="good", folders={Folder.SSL})
    assert PartialBackupOptions(name="good", homeassistant=True)
    with pytest.raises(
        ValueError,
        match="At least one of addons, folders, or homeassistant must have a value",
    ):
        PartialBackupOptions(name="bad")


async def test_partial_restore_options() -> None:
    """Test partial restore options."""
    assert PartialRestoreOptions(addons={"a"})
    assert PartialRestoreOptions(folders={Folder.SSL})
    assert PartialRestoreOptions(homeassistant=True)
    with pytest.raises(
        ValueError,
        match="At least one of addons, folders, or homeassistant must have a value",
    ):
        PartialRestoreOptions(background=True)


async def test_backup_options_location() -> None:
    """Test location field in backup options."""
    assert FullBackupOptions(location=["test", None]).to_dict() == {
        "location": ["test", None]
    }
    assert FullBackupOptions(location="test").to_dict() == {"location": "test"}
    assert FullBackupOptions().to_dict() == {}

    assert PartialBackupOptions(
        location=["test", ".local"], folders={Folder.SSL}
    ).to_dict() == {
        "location": ["test", ".local"],
        "folders": ["ssl"],
    }
    assert PartialBackupOptions(location="test", folders={Folder.SSL}).to_dict() == {
        "location": "test",
        "folders": ["ssl"],
    }
    assert PartialBackupOptions(folders={Folder.SSL}).to_dict() == {"folders": ["ssl"]}


def backup_callback(url: str, **kwargs: dict[str, Any]) -> CallbackResult:  # noqa: ARG001
    """Return response based on whether backup was in background or not."""
    if kwargs["json"] and kwargs["json"].get("background"):
        fixture = "backup_background.json"
    else:
        fixture = "backup_foreground.json"
    return CallbackResult(status=200, body=load_fixture(fixture))


@pytest.mark.parametrize(
    ("options", "slug"),
    [
        (FullBackupOptions(name="Test", background=True), None),
        (FullBackupOptions(name="Test", background=False), "9ecf0028"),
        (FullBackupOptions(name="Test", background=False, location="test"), "9ecf0028"),
        (
            FullBackupOptions(
                name="Test", background=False, location={".local", "test"}
            ),
            "9ecf0028",
        ),
        (
            FullBackupOptions(
                name="Test", background=False, extra={"user": "test", "scheduled": True}
            ),
            "9ecf0028",
        ),
        (FullBackupOptions(name="Test", background=False, extra=None), "9ecf0028"),
        (
            FullBackupOptions(
                name="test", background=False, filename=PurePath("backup.tar")
            ),
            "9ecf0028",
        ),
        (None, "9ecf0028"),
    ],
)
async def test_backups_full_backup(
    responses: aioresponses,
    supervisor_client: SupervisorClient,
    options: FullBackupOptions | None,
    slug: str | None,
) -> None:
    """Test backups full backup API."""
    responses.post(
        f"{SUPERVISOR_URL}/backups/new/full",
        callback=backup_callback,
    )
    result = await supervisor_client.backups.full_backup(options)
    assert result.job_id.hex == "dc9dbc16f6ad4de592ffa72c807ca2bf"
    assert result.slug == slug


@pytest.mark.parametrize(
    ("options", "slug"),
    [
        (PartialBackupOptions(name="Test", background=True, addons={"core_ssh"}), None),
        (
            PartialBackupOptions(name="Test", background=False, addons={"core_ssh"}),
            "9ecf0028",
        ),
        (
            PartialBackupOptions(
                name="Test", background=False, location="test", addons={"core_ssh"}
            ),
            "9ecf0028",
        ),
        (
            PartialBackupOptions(
                name="Test",
                background=False,
                location={".local", "test"},
                addons={"core_ssh"},
            ),
            "9ecf0028",
        ),
        (
            PartialBackupOptions(
                name="Test",
                background=False,
                addons={"core_ssh"},
                extra={"user": "test", "scheduled": True},
            ),
            "9ecf0028",
        ),
        (
            PartialBackupOptions(
                name="Test", background=False, addons={"core_ssh"}, extra=None
            ),
            "9ecf0028",
        ),
        (
            PartialBackupOptions(
                name="Test",
                background=False,
                addons={"core_ssh"},
                filename=PurePath("backup.tar"),
            ),
            "9ecf0028",
        ),
        (
            PartialBackupOptions(name="Test", background=None, addons={"core_ssh"}),
            "9ecf0028",
        ),
        (
            PartialBackupOptions(name="Test", background=None, addons=AddonSet.ALL),
            "9ecf0028",
        ),
    ],
)
async def test_backups_partial_backup(
    responses: aioresponses,
    supervisor_client: SupervisorClient,
    options: PartialBackupOptions,
    slug: str | None,
) -> None:
    """Test backups full backup API."""
    responses.post(
        f"{SUPERVISOR_URL}/backups/new/partial",
        callback=backup_callback,
    )
    result = await supervisor_client.backups.partial_backup(options)
    assert result.job_id.hex == "dc9dbc16f6ad4de592ffa72c807ca2bf"
    assert result.slug == slug


async def test_backup_info(
    responses: aioresponses, supervisor_client: SupervisorClient
) -> None:
    """Test backup info API."""
    responses.get(
        f"{SUPERVISOR_URL}/backups/69558789/info",
        status=200,
        body=load_fixture("backup_info.json"),
    )
    result = await supervisor_client.backups.backup_info("69558789")
    assert result.slug == "69558789"
    assert result.type == "partial"
    assert result.date == datetime(2024, 5, 31, 0, 0, 0, 0, UTC)
    assert result.compressed is True
    assert result.addons[0].slug == "core_mosquitto"
    assert result.addons[0].name == "Mosquitto broker"
    assert result.addons[0].version == "6.4.0"
    assert result.addons[0].size == 0
    assert result.repositories == [
        "core",
        "local",
        "https://github.com/music-assistant/home-assistant-addon",
        "https://github.com/esphome/home-assistant-addon",
        "https://github.com/hassio-addons/repository",
    ]
    assert result.folders == []
    assert result.homeassistant_exclude_database is None
    assert result.extra is None
    assert result.location_attributes[".local"].protected is False
    assert result.location_attributes[".local"].size_bytes == 10123


async def test_backup_info_no_homeassistant(
    responses: aioresponses, supervisor_client: SupervisorClient
) -> None:
    """Test backup info API with no home assistant."""
    responses.get(
        f"{SUPERVISOR_URL}/backups/d13dedd0/info",
        status=200,
        body=load_fixture("backup_info_no_homeassistant.json"),
    )
    result = await supervisor_client.backups.backup_info("d13dedd0")
    assert result.slug == "d13dedd0"
    assert result.type == "partial"
    assert result.homeassistant is None


async def test_backup_info_with_extra(
    responses: aioresponses, supervisor_client: SupervisorClient
) -> None:
    """Test backup info API with extras set by client."""
    responses.get(
        f"{SUPERVISOR_URL}/backups/d13dedd0/info",
        status=200,
        body=load_fixture("backup_info_with_extra.json"),
    )
    result = await supervisor_client.backups.backup_info("d13dedd0")
    assert result.slug == "69558789"
    assert result.type == "partial"
    assert result.extra == {"user": "test", "scheduled": True}


async def test_backup_info_with_multiple_locations(
    responses: aioresponses, supervisor_client: SupervisorClient
) -> None:
    """Test backup info API with multiple locations."""
    responses.get(
        f"{SUPERVISOR_URL}/backups/d13dedd0/info",
        status=200,
        body=load_fixture("backup_info_with_locations.json"),
    )
    result = await supervisor_client.backups.backup_info("d13dedd0")
    assert result.slug == "69558789"
    assert result.type == "partial"
    assert result.location_attributes[".local"].protected is False
    assert result.location_attributes[".local"].size_bytes == 10123
    assert result.location_attributes["Test"].protected is False
    assert result.location_attributes["Test"].size_bytes == 10123


@pytest.mark.parametrize(
    "options", [None, RemoveBackupOptions(location={"test", None})]
)
async def test_remove_backup(
    responses: aioresponses,
    supervisor_client: SupervisorClient,
    options: RemoveBackupOptions | None,
) -> None:
    """Test remove backup API."""
    responses.delete(f"{SUPERVISOR_URL}/backups/abc123", status=200)
    assert await supervisor_client.backups.remove_backup("abc123", options) is None
    assert responses.requests.keys() == {
        ("DELETE", URL(f"{SUPERVISOR_URL}/backups/abc123"))
    }


@pytest.mark.parametrize(
    "options",
    [
        None,
        FullRestoreOptions(password="abc123"),  # noqa: S106
        FullRestoreOptions(background=True),
        FullRestoreOptions(location="test"),
    ],
)
async def test_full_restore(
    responses: aioresponses,
    supervisor_client: SupervisorClient,
    options: FullRestoreOptions | None,
) -> None:
    """Test full restore API."""
    responses.post(
        f"{SUPERVISOR_URL}/backups/abc123/restore/full",
        status=200,
        body=load_fixture("backup_restore.json"),
    )
    result = await supervisor_client.backups.full_restore("abc123", options)
    assert result.job_id.hex == "dc9dbc16f6ad4de592ffa72c807ca2bf"


@pytest.mark.parametrize(
    "options",
    [
        PartialRestoreOptions(addons={"core_ssh"}),
        PartialRestoreOptions(homeassistant=True, location=".local"),
        PartialRestoreOptions(folders={Folder.SHARE, Folder.SSL}, location="test"),
        PartialRestoreOptions(addons={"core_ssh"}, background=True),
        PartialRestoreOptions(addons={"core_ssh"}, password="abc123"),  # noqa: S106
    ],
)
async def test_partial_restore(
    responses: aioresponses,
    supervisor_client: SupervisorClient,
    options: PartialRestoreOptions,
) -> None:
    """Test partial restore API."""
    responses.post(
        f"{SUPERVISOR_URL}/backups/abc123/restore/partial",
        status=200,
        body=load_fixture("backup_restore.json"),
    )
    result = await supervisor_client.backups.partial_restore("abc123", options)
    assert result.job_id.hex == "dc9dbc16f6ad4de592ffa72c807ca2bf"


@pytest.mark.parametrize(
    ("options", "query"),
    [
        (None, ""),
        (
            UploadBackupOptions(location={".local", "test"}),
            "?location=.local&location=test",
        ),
        (UploadBackupOptions(filename=PurePath("backup.tar")), "?filename=backup.tar"),
    ],
)
async def test_upload_backup(
    responses: aioresponses,
    supervisor_client: SupervisorClient,
    options: UploadBackupOptions | None,
    query: str,
) -> None:
    """Test upload backup API."""
    responses.post(
        f"{SUPERVISOR_URL}/backups/new/upload{query}",
        status=200,
        body=load_fixture("backup_uploaded.json"),
    )
    data = asyncio.StreamReader(loop=asyncio.get_running_loop())
    data.feed_data(b"backup test")
    data.feed_eof()

    result = await supervisor_client.backups.upload_backup(data, options)
    assert result == "7fed74c8"


@pytest.mark.parametrize(
    ("options", "query"),
    [
        (None, ""),
        (DownloadBackupOptions(location="test"), "?location=test"),
        (DownloadBackupOptions(location=None), "?location="),
    ],
)
async def test_download_backup(
    responses: aioresponses,
    supervisor_client: SupervisorClient,
    options: DownloadBackupOptions | None,
    query: str,
) -> None:
    """Test download backup API."""
    responses.get(
        f"{SUPERVISOR_URL}/backups/7fed74c8/download{query}",
        status=200,
        body=b"backup test",
    )
    result = await supervisor_client.backups.download_backup("7fed74c8", options)
    assert isinstance(result, AsyncIterator)
    async for chunk in result:
        assert chunk == b"backup test"


@pytest.mark.parametrize(
    ("options", "as_dict"),
    [
        (
            PartialBackupOptions(name="Test", folders={Folder.SHARE}),
            {"name": "Test", "folders": ["share"]},
        ),
        (PartialBackupOptions(addons={"core_ssh"}), {"addons": ["core_ssh"]}),
        (PartialBackupOptions(addons=AddonSet.ALL), {"addons": "ALL"}),
        (
            PartialBackupOptions(
                homeassistant=True, homeassistant_exclude_database=True
            ),
            {"homeassistant": True, "homeassistant_exclude_database": True},
        ),
        (
            PartialBackupOptions(
                folders={Folder.SSL}, compressed=True, background=True
            ),
            {"folders": ["ssl"], "compressed": True, "background": True},
        ),
        (
            PartialBackupOptions(
                homeassistant=True, location=[".cloud_backup", "test"]
            ),
            {"homeassistant": True, "location": [".cloud_backup", "test"]},
        ),
        (
            PartialBackupOptions(homeassistant=True, location="test"),
            {"homeassistant": True, "location": "test"},
        ),
        (
            PartialBackupOptions(homeassistant=True, filename=PurePath("backup.tar")),
            {"homeassistant": True, "filename": "backup.tar"},
        ),
    ],
)
async def test_partial_backup_model(
    options: PartialBackupOptions, as_dict: dict[str, Any]
) -> None:
    """Test partial backup model parsing and serializing."""
    assert PartialBackupOptions.from_dict(as_dict) == options
    assert options.to_dict() == as_dict


@pytest.mark.parametrize(
    ("options", "as_dict"),
    [
        (FullBackupOptions(name="Test"), {"name": "Test"}),
        (FullBackupOptions(password="test"), {"password": "test"}),  # noqa: S106
        (FullBackupOptions(compressed=True), {"compressed": True}),
        (
            FullBackupOptions(homeassistant_exclude_database=True),
            {"homeassistant_exclude_database": True},
        ),
        (FullBackupOptions(background=True), {"background": True}),
        (
            FullBackupOptions(location=[".cloud_backup", "test"]),
            {"location": [".cloud_backup", "test"]},
        ),
        (FullBackupOptions(location="test"), {"location": "test"}),
        (
            FullBackupOptions(filename=PurePath("backup.tar")),
            {"filename": "backup.tar"},
        ),
    ],
)
async def test_full_backup_model(
    options: FullBackupOptions, as_dict: dict[str, Any]
) -> None:
    """Test full backup model parsing and serializing."""
    assert FullBackupOptions.from_dict(as_dict) == options
    assert options.to_dict() == as_dict


async def test_backups_list_location_attributes(
    responses: aioresponses,
    supervisor_client: SupervisorClient,
) -> None:
    """Test location attributes field in backups list."""
    responses.get(
        f"{SUPERVISOR_URL}/backups",
        status=200,
        body=load_fixture("backups_list_location_attributes.json"),
    )

    result = await supervisor_client.backups.list()
    assert result[0].location_attributes == {
        ".local": BackupLocationAttributes(
            protected=True,
            size_bytes=10240,
        ),
        "test": BackupLocationAttributes(
            protected=True,
            size_bytes=10240,
        ),
    }


async def test_backup_info_location_attributes(
    responses: aioresponses,
    supervisor_client: SupervisorClient,
) -> None:
    """Test location attributes field in backup info."""
    responses.get(
        f"{SUPERVISOR_URL}/backups/d9c48f8b/info",
        status=200,
        body=load_fixture("backup_info_location_attributes.json"),
    )

    result = await supervisor_client.backups.backup_info("d9c48f8b")
    assert result.location_attributes == {
        ".local": BackupLocationAttributes(
            protected=True,
            size_bytes=10240,
        ),
        "test": BackupLocationAttributes(
            protected=True,
            size_bytes=10240,
        ),
    }



"""Exceptions from supervisor client."""


class SupervisorError(Exception):
    """Generic exception."""

    def __init__(self, message: str | None = None, job_id: str | None = None) -> None:
        """Initialize exception."""
        if message is not None:
            super().__init__(message)
        else:
            super().__init__()

        self.job_id: str | None = job_id


class SupervisorConnectionError(SupervisorError, ConnectionError):
    """Unknown error connecting to supervisor."""


class SupervisorTimeoutError(SupervisorError, TimeoutError):
    """Timeout connecting to supervisor."""


class SupervisorBadRequestError(SupervisorError):
    """Invalid request made to supervisor."""


class SupervisorAuthenticationError(SupervisorError):
    """Invalid authentication sent to supervisor."""


class SupervisorForbiddenError(SupervisorError):
    """Client is not allowed to take the action requested."""


class SupervisorNotFoundError(SupervisorError):
    """Requested resource does not exist."""


class SupervisorServiceUnavailableError(SupervisorError):
    """Cannot complete request because a required service is unavailable."""


class SupervisorResponseError(SupervisorError):
    """Unusable response received from Supervisor with the wrong type or encoding."""



"""Models for Supervisor backups."""

from abc import ABC
from dataclasses import dataclass
from datetime import datetime
from enum import StrEnum
from pathlib import PurePath
from uuid import UUID

from .base import Options, Request, ResponseData

LOCATION_LOCAL_STORAGE = ".local"
LOCATION_CLOUD_BACKUP = ".cloud_backup"

# --- ENUMS ----


class BackupType(StrEnum):
    """BackupType type."""

    FULL = "full"
    PARTIAL = "partial"


class Folder(StrEnum):
    """Folder type."""

    SHARE = "share"
    ADDONS = "addons/local"
    SSL = "ssl"
    MEDIA = "media"


class AddonSet(StrEnum):
    """AddonSet type."""

    ALL = "ALL"


# --- OBJECTS ----


@dataclass(frozen=True, slots=True)
class BackupContent(ResponseData):
    """BackupContent model."""

    homeassistant: bool
    addons: list[str]
    folders: list[Folder]


@dataclass(frozen=True, slots=True)
class BackupLocationAttributes(ResponseData):
    """BackupLocationAttributes model."""

    protected: bool
    size_bytes: int


@dataclass(frozen=True)
class BackupBaseFields(ABC):
    """BackupBaseFields ABC type."""

    slug: str
    name: str
    date: datetime
    type: BackupType
    location_attributes: dict[str, BackupLocationAttributes]
    compressed: bool


@dataclass(frozen=True, slots=True)
class Backup(BackupBaseFields, ResponseData):
    """Backup model."""

    content: BackupContent


@dataclass(frozen=True, slots=True)
class BackupAddon(ResponseData):
    """BackupAddon model."""

    slug: str
    name: str
    version: str
    size: float


@dataclass(frozen=True, slots=True)
class BackupComplete(BackupBaseFields, ResponseData):
    """BackupComplete model."""

    supervisor_version: str
    homeassistant: str | None
    addons: list[BackupAddon]
    repositories: list[str]
    folders: list[Folder]
    homeassistant_exclude_database: bool | None
    extra: dict | None


@dataclass(frozen=True, slots=True)
class BackupList(ResponseData):
    """BackupList model."""

    backups: list[Backup]


@dataclass(frozen=True, slots=True)
class BackupsInfo(BackupList):
    """BackupsInfo model."""

    days_until_stale: int


@dataclass(frozen=True, slots=True)
class BackupsOptions(Request):
    """BackupsOptions model."""

    days_until_stale: int


@dataclass(frozen=True, slots=True)
class FreezeOptions(Request):
    """FreezeOptions model."""

    timeout: int


@dataclass(frozen=True)
class PartialBackupRestoreOptions(ABC):  # noqa: B024
    """PartialBackupRestoreOptions ABC type."""

    addons: set[str] | None = None
    folders: set[Folder] | None = None
    homeassistant: bool | None = None

    def __post_init__(self) -> None:
        """Validate at least one thing to backup/restore is included."""
        if not any((self.addons, self.folders, self.homeassistant)):
            raise ValueError(
                "At least one of addons, folders, or homeassistant must have a value"
            )


@dataclass(frozen=True, slots=True)
class FullBackupOptions(Request):
    """FullBackupOptions model."""

    name: str | None = None
    password: str | None = None
    compressed: bool | None = None
    location: str | list[str] = None  # type: ignore[assignment]
    homeassistant_exclude_database: bool | None = None
    background: bool | None = None
    extra: dict | None = None
    filename: PurePath | None = None


@dataclass(frozen=True, slots=True)
class PartialBackupOptions(FullBackupOptions, PartialBackupRestoreOptions):
    """PartialBackupOptions model."""

    addons: AddonSet | set[str] | None = None


@dataclass(frozen=True, slots=True)
class BackupJob(ResponseData):
    """BackupJob model."""

    job_id: UUID


@dataclass(frozen=True, slots=True)
class NewBackup(BackupJob):
    """NewBackup model."""

    slug: str | None = None


@dataclass(frozen=True, slots=True)
class FullRestoreOptions(Request):
    """FullRestoreOptions model."""

    password: str | None = None
    background: bool | None = None
    location: str = None  # type: ignore[assignment]


@dataclass(frozen=True, slots=True)
class PartialRestoreOptions(FullRestoreOptions, PartialBackupRestoreOptions):
    """PartialRestoreOptions model."""


@dataclass(frozen=True, slots=True)
class UploadBackupOptions(Options):
    """UploadBackupOptions model."""

    location: set[str] = None
    filename: PurePath | None = None


@dataclass(frozen=True, slots=True)
class UploadedBackup(ResponseData):
    """UploadedBackup model."""

    slug: str


@dataclass(frozen=True, slots=True)
class RemoveBackupOptions(Request):
    """RemoveBackupOptions model."""

    location: set[str] = None


@dataclass(frozen=True, slots=True)
class DownloadBackupOptions(Request):
    """DownloadBackupOptions model."""

    location: str = None  # type: ignore[assignment]
